@@ -1,13 +1,13 @@
 /*
  * Copyright 2003-2007 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanelf.c,v 1.189 2008/01/17 04:37:19 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanelf.c,v 1.196 2008/10/22 17:03:17 flameeyes Exp $
  *
  * Copyright 2003-2007 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2004-2007 Mike Frysinger  - <vapier@gentoo.org>
  */
 
-static const char *rcsid = "$Id: scanelf.c,v 1.189 2008/01/17 04:37:19 solar Exp $";
+static const char *rcsid = "$Id: scanelf.c,v 1.196 2008/10/22 17:03:17 flameeyes Exp $";
 const char * const argv0 = "scanelf";
 
 #include "paxinc.h"
@@ -40,6 +40,7 @@ static char dir_recurse = 0;
 static char dir_crossmount = 1;
 static char show_pax = 0;
 static char show_perms = 0;
+static char show_size = 0;
 static char show_phdr = 0;
 static char show_textrel = 0;
 static char show_rpath = 0;
@@ -50,6 +51,8 @@ static char show_soname = 0;
 static char show_textrels = 0;
 static char show_banner = 1;
 static char show_endian = 0;
+static char show_osabi = 0;
+static char show_eabi = 0;
 static char be_quiet = 0;
 static char be_verbose = 0;
 static char be_wewy_wewy_quiet = 0;
@@ -793,7 +796,9 @@ static char *lookup_cache_lib(elfobj *elf, char *fname)
 	return NULL;
 }
 #else
+#ifdef __ELF__
 #warning Cache support not implemented for your target
+#endif
 static char *lookup_cache_lib(elfobj *elf, char *fname)
 {
 	return NULL;
@@ -994,6 +999,10 @@ static char *scanelf_file_sym(elfobj *elf, char *found_sym)
 		if (cnt) \
 			cnt = EGET(symtab->sh_size) / cnt; \
 		for (i = 0; i < cnt; ++i) { \
+			if ((void*)sym > (void*)elf->data_end) { \
+				warnf("%s: corrupt ELF symbols - aborting", elf->filename); \
+				goto break_out;	\
+			} \
 			if (sym->st_name) { \
 				/* make sure the symbol name is in acceptable memory range */ \
 				symname = (char *)(elf->data + EGET(strtab->sh_offset) + EGET(sym->st_name)); \
@@ -1146,12 +1155,15 @@ static int scanelf_elfobj(elfobj *elf)
 			case 'n': prints("NEEDED "); break;
 			case 'i': prints("INTERP "); break;
 			case 'b': prints("BIND "); break;
+			case 'Z': prints("SIZE "); break;
 			case 'S': prints("SONAME "); break;
 			case 's': prints("SYM "); break;
 			case 'N': prints("LIB "); break;
 			case 'T': prints("TEXTRELS "); break;
 			case 'k': prints("SECTION "); break;
 			case 'a': prints("ARCH "); break;
+			case 'I': prints("OSABI "); break;
+			case 'Y': prints("EABI "); break;
 			case 'O': prints("PERM "); break;
 			case 'D': prints("ENDIAN "); break;
 			default: warnf("'%c' has no title ?", out_format[i]);
@@ -1167,7 +1179,7 @@ static int scanelf_elfobj(elfobj *elf)
 	for (i = 0; out_format[i]; ++i) {
 		const char *out;
 		const char *tmp;
-
+		static char ubuf[sizeof(unsigned long)*2];
 		if (!IS_MODIFIER(out_format[i])) {
 			xchrcat(&out_buffer, out_format[i], &out_len);
 			continue;
@@ -1224,6 +1236,9 @@ static int scanelf_elfobj(elfobj *elf)
 		case 's': out = scanelf_file_sym(elf, &found_sym); break;
 		case 'k': out = scanelf_file_sections(elf, &found_section); break;
 		case 'a': out = get_elfemtype(elf); break;
+		case 'I': out = get_elfosabi(elf); break;
+		case 'Y': out = get_elf_eabi(elf); break;
+		case 'Z': snprintf(ubuf, sizeof(ubuf), "%lu", (unsigned long)elf->len); out = ubuf; break;;
 		default: warnf("'%c' has no scan code?", out_format[i]);
 		}
 		if (out) {
@@ -1537,13 +1552,14 @@ static int load_ld_cache_config(int i, const char *fname)
 }
 
 #else
-
+#ifdef __ELF__
 #warning Cache config support not implemented for your target
+#endif
 static int load_ld_cache_config(int i, const char *fname)
 {
 	memset(ldpaths, 0x00, sizeof(ldpaths));
+	return 0;
 }
-
 #endif
 
 /* scan /etc/ld.so.conf for paths */
@@ -1588,8 +1604,8 @@ static void scanelf_envpath(void)
 	free(path);
 }
 
-/* usage / invocation handling functions */ /* Free Flags: c d j u w C G H I J K P Q U W Y Z */
-#define PARSE_FLAGS "plRmyAXz:xetrnLibSs:k:gN:TaqvF:f:o:E:M:DO:BhV"
+/* usage / invocation handling functions */ /* Free Flags: c d j u w C G H J K P Q U W */
+#define PARSE_FLAGS "plRmyAXz:xetrnLibSs:k:gN:TaqvF:f:o:E:M:DIYO:ZBhV"
 #define a_argument required_argument
 static struct option const long_opts[] = {
 	{"path",      no_argument, NULL, 'p'},
@@ -1617,7 +1633,10 @@ static struct option const long_opts[] = {
 	{"etype",      a_argument, NULL, 'E'},
 	{"bits",       a_argument, NULL, 'M'},
 	{"endian",    no_argument, NULL, 'D'},
+	{"osabi",     no_argument, NULL, 'I'},
+	{"eabi",      no_argument, NULL, 'Y'},
 	{"perms",      a_argument, NULL, 'O'},
+	{"size",      no_argument, NULL, 'Z'},
 	{"all",       no_argument, NULL, 'a'},
 	{"quiet",     no_argument, NULL, 'q'},
 	{"verbose",   no_argument, NULL, 'v'},
@@ -1656,7 +1675,10 @@ static const char *opts_help[] = {
 	"Print only ELF files matching etype ET_DYN,ET_EXEC ...",
 	"Print only ELF files matching numeric bits",
 	"Print Endianness",
+	"Print OSABI",
+	"Print EABI (EM_ARM Only)",
 	"Print only ELF files matching octal permissions",
+	"Print ELF file size",
 	"Print all scanned info (-x -e -t -r -b)\n",
 	"Only output 'bad' things",
 	"Be verbose (can be specified more than once)",
@@ -1804,6 +1826,7 @@ static int parseargs(int argc, char *argv[])
 					setpax = flags;
 			break;
 		}
+		case 'Z': show_size = 1; break;
 		case 'g': g_match = 1; break;
 		case 'L': use_ldcache = 1; break;
 		case 'y': scan_symlink = 0; break;
@@ -1827,6 +1850,8 @@ static int parseargs(int argc, char *argv[])
 		case 'v': be_verbose = (be_verbose % 20) + 1; break;
 		case 'a': show_perms = show_pax = show_phdr = show_textrel = show_rpath = show_bind = show_endian = 1; break;
 		case 'D': show_endian = 1; break;
+		case 'I': show_osabi = 1; break;
+		case 'Y': show_eabi = 1; break;
 		case ':':
 			err("Option '%c' is missing parameter", optopt);
 		case '?':
@@ -1843,7 +1868,8 @@ static int parseargs(int argc, char *argv[])
 	if (out_format) {
 		show_pax = show_phdr = show_textrel = show_rpath = \
 		show_needed = show_interp = show_bind = show_soname = \
-		show_textrels = show_perms = show_endian = 0;
+		show_textrels = show_perms = show_endian = show_size = \
+		show_osabi = show_eabi = 0;
 		for (i = 0; out_format[i]; ++i) {
 			if (!IS_MODIFIER(out_format[i])) continue;
 
@@ -1860,7 +1886,10 @@ static int parseargs(int argc, char *argv[])
 			case 'o': break;
 			case 'a': break;
 			case 'M': break;
+			case 'Z': show_size = 1; break;
 			case 'D': show_endian = 1; break;
+			case 'I': show_osabi = 1; break;
+			case 'Y': show_eabi = 1; break;
 			case 'O': show_perms = 1; break;
 			case 'x': show_pax = 1; break;
 			case 'e': show_phdr = 1; break;
@@ -1881,10 +1910,14 @@ static int parseargs(int argc, char *argv[])
 	} else {
 		size_t fmt_len = 30;
 		out_format = xmalloc(sizeof(char) * fmt_len);
+		*out_format = '\0';
 		if (!be_quiet)     xstrcat(&out_format, "%o ", &fmt_len);
 		if (show_pax)      xstrcat(&out_format, "%x ", &fmt_len);
 		if (show_perms)    xstrcat(&out_format, "%O ", &fmt_len);
+		if (show_size)     xstrcat(&out_format, "%Z ", &fmt_len);
 		if (show_endian)   xstrcat(&out_format, "%D ", &fmt_len);
+		if (show_osabi)    xstrcat(&out_format, "%I ", &fmt_len);
+		if (show_eabi)     xstrcat(&out_format, "%Y ", &fmt_len);
 		if (show_phdr)     xstrcat(&out_format, "%e ", &fmt_len);
 		if (show_textrel)  xstrcat(&out_format, "%t ", &fmt_len);
 		if (show_rpath)    xstrcat(&out_format, "%r ", &fmt_len);
